////////////////////////////////////////////////////////////////////////////////
//Dan Nielsen
////////////////////////////////////////////////////////////////////////////////

#ifndef NOISE1D_CPP
#define NOISE1D_CPP

#include "Noise1D.h"

void Noise1D::init() {
  for(int i=0;i<256;i++) {
    grad[i]=commonPermute256[i];
  }
}

void Noise1D::close() {
}

void Noise1D::update() {
  //Scalar update
  for(int i=0;i<256;i++) {
    if ((grad[i]&0x7)==0x7) { grad[i]&=0x7f; }
    else if ((grad[i]&0x7)==0x0) { grad[i]|=0x80; }
    grad[i]+=commonRandomBool()*(((grad[i]&0x80)>>6)-1);
  }

  //Gradient swap
  bool k=false;
  for(int i=0,j=0;i<256;i++) {
    if ((grad[i]&0x7)==0) {
      if (k) {
        int tmp=grad[i];
        grad[i]=grad[j];
        grad[j]=tmp;
      }
      else {
        k=true;
      }
      j=i;
    }
  }
}

int Noise1D::get(int x,int xiMask) {
  int g0=grad[(x>>8)&xiMask],
      g1=grad[((x>>8)+1)&xiMask];

  //Quick dot product; scaling; S-curve; interpolation
  return mmInterp(gridPtEval(g0,(x&0xff)),
                  gridPtEval(g1,(x&0xff)|0xffffff00),
                  mmSCurve(x&0xff));
}

int Noise1D::gridPtEval(int g,int dx) {
  return ((g&0x08)>0? -dx : dx)*(g&0x07);
}

#endif
