////////////////////////////////////////////////////////////////////////////////
//Dan Nielsen
////////////////////////////////////////////////////////////////////////////////

#ifndef NOISE2D_CPP
#define NOISE2D_CPP

#include "Noise2D.h"

void Noise2D::init() {
  for(int i=0;i<256;i++) {
    grad[i]=commonPermute256[i];
  }
}

void Noise2D::close() {
}

void Noise2D::update() {
  //Scalar update
  for(int i=0;i<256;i++) {
    if ((grad[i]&0x7)==0x7) { grad[i]&=0x7f; }
    else if ((grad[i]&0x7)==0x0) { grad[i]|=0x80; }
    grad[i]+=commonRandomBool()*(((grad[i]&0x80)>>6)-1);
  }

  //Gradient swap
  bool k=false;
  for(int i=0,j=0;i<256;i++) {
    if ((grad[i]&0x7)==0) {
      if (k) {
        int tmp=grad[i];
        grad[i]=grad[j];
        grad[j]=tmp;
      }
      else {
        k=true;
      }
      j=i;
    }
  }
}

int Noise2D::get(int x,int y,int xiMask,int yiMask) {
  int h[2],
      g[4];

  //Permutation
  h[0]=commonPermute256[(y>>8)&yiMask];
  h[1]=commonPermute256[((y>>8)+1)&yiMask];
  int xi=(x>>8)&xiMask,
      x1=(xi+1)&xiMask;
  g[0]=grad[xi^h[0]]; //Novelty: XOR instead of modulo addition
  g[1]=grad[x1^h[0]];
  g[2]=grad[xi^h[1]];
  g[3]=grad[x1^h[1]];

  //Quick dot product; scaling; S-curve; interpolation
  int xf=x&0xff,yf=y&0xff;
  int sx=mmSCurve(xf);
  return mmInterp(mmInterp(gridPtEval(g[0],xf,yf),
                           gridPtEval(g[1],xf|0xffffff00,yf),
                           sx),
                  mmInterp(gridPtEval(g[2],xf,yf|0xffffff00),
                           gridPtEval(g[3],xf|0xffffff00,yf|0xffffff00),
                           sx),
                  mmSCurve(yf));
}

int Noise2D::gridPtEval(int g,int dx,int dy) {
  int pa,pb;

  if ((g&0x08)>0) { pa=dx; pb=dy; }
  else { pa=dy; pb=dx; }

  return ((((g&0x10)>0? -pa : pa)<<1)+((g&0x20)>0? -pb : pb))*(g&0x07);
}

#endif
