////////////////////////////////////////////////////////////////////////////////
//Dan Nielsen
////////////////////////////////////////////////////////////////////////////////

#ifndef NOISE3D_CPP
#define NOISE3D_CPP

#include "Noise3D.h"

void Noise3D::init() {
  //Gradient vectors:
  //[ 1  1  0] [ 1  1  0] [-1  1  0] [-1  1  0]
  //[ 1 -1  0] [-1 -1  0] [ 1  0  1] [-1  0  1]
  //[ 0  1  1] [ 0 -1  1] [ 0 -1  1] [ 1  0 -1]
  //[-1  0 -1] [ 0  1 -1] [ 0 -1 -1] [ 0 -1 -1] <-Duplicated tetrahedron
  //                                              for a total 16 vectors
  //                                              (as Perlin suggested)
  for(int i=0;i<256;i++) {
    grad[i]=commonPermute256[i];
  }
}

void Noise3D::close() {
}

void Noise3D::update() {
  //Scalar update
  for(int i=0;i<256;i++) {
    if ((grad[i]&0x7)==0x7) { grad[i]&=0x7f; }
    else if ((grad[i]&0x7)==0x0) { grad[i]|=0x80; }
    grad[i]+=commonRandomBool()*(((grad[i]&0x80)>>6)-1);
  }

  //Gradient swap
  bool k=false;
  for(int i=0,j=0;i<256;i++) {
    if ((grad[i]&0x7)==0) {
      if (k) {
        int tmp=grad[i];
        grad[i]=grad[j];
        grad[j]=tmp;
      }
      else {
        k=true;
      }
      j=i;
    }
  }
}

int Noise3D::get(int x,int y,int z,int xiMask,int yiMask,int ziMask) {
  int q[2],
      h[4],
      g[8];

  //Permutation
  q[0]=commonPermute256[(z>>8)&ziMask];
  q[1]=commonPermute256[((z>>8)+1)&ziMask];
  int yi=(y>>8)&yiMask,
      y1=(yi+1)&yiMask;
  h[0]=commonPermute256[q[0]^yi]; //Novelty: XOR instead of modulo addition
  h[1]=commonPermute256[q[0]^y1];
  h[2]=commonPermute256[q[1]^yi];
  h[3]=commonPermute256[q[1]^y1];
  int xi=(x>>8)&xiMask,
      x1=(xi+1)&xiMask;
  g[0]=grad[xi^h[0]];
  g[1]=grad[x1^h[0]];
  g[2]=grad[xi^h[1]];
  g[3]=grad[x1^h[1]];
  g[4]=grad[xi^h[2]];
  g[5]=grad[x1^h[2]];
  g[6]=grad[xi^h[3]];
  g[7]=grad[x1^h[3]];

  //Quick dot product; scaling; S-curve; interpolation
  int xf=x&0xff,yf=y&0xff,zf=z&0xff;
  int sx=mmSCurve(xf),sy=mmSCurve(yf);
  return mmInterp(mmInterp(mmInterp(gridPtEval(g[0],xf,yf,zf),
                                    gridPtEval(g[1],xf|0xffffff00,yf,zf),
                                    sx),
                           mmInterp(gridPtEval(g[2],xf,yf|0xffffff00,zf),
                                    gridPtEval(g[3],xf|0xffffff00,yf|0xffffff00,zf),
                                    sx),
                           sy),
                  mmInterp(mmInterp(gridPtEval(g[4],xf,yf,zf|0xffffff00),
                                    gridPtEval(g[5],xf|0xffffff00,yf,zf|0xffffff00),
                                    sx),
                           mmInterp(gridPtEval(g[6],xf,yf|0xffffff00,zf|0xffffff00),
                                    gridPtEval(g[7],xf|0xffffff00,yf|0xffffff00,zf|0xffffff00),
                                    sx),
                           sy),
                  mmSCurve(zf));
}

int Noise3D::gridPtEval(int g,int dx,int dy,int dz) {
      int h=(g>>3)&0xf;
      int u=(((h&0x10)==0)? dx : dy),
          v=h<4? dy : h==12||h==14? dx : dz;
      return (((h&1)==0? u : -u)+((h&2)==0? v : -v))*(g&0x7);
}

#endif
